package com.ylx.web.controller.massage;

import cn.hutool.core.io.FileUtil;
import cn.hutool.extra.qrcode.QrCodeUtil;
import cn.hutool.extra.qrcode.QrConfig;
import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ylx.common.annotation.Log;
import com.ylx.common.constant.Constants;
import com.ylx.common.core.domain.R;
import com.ylx.common.core.domain.model.WxLoginUser;
import com.ylx.common.enums.BusinessType;
import com.ylx.common.utils.MessageUtils;
import com.ylx.framework.manager.AsyncManager;
import com.ylx.framework.manager.factory.AsyncFactory;
import com.ylx.framework.web.service.WxTokenService;
import com.ylx.massage.domain.CouponReceive;
import com.ylx.massage.domain.TWxUser;
import com.ylx.massage.service.CouponReceiveService;
import com.ylx.massage.service.TWxUserService;
import com.ylx.massage.utils.WeChatUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.Map;

import static com.ylx.massage.utils.OtherUtil.verification;

/**
 * @author b16mt
 */
@Slf4j
@RestController
@Api(tags = {"微信公众号"})
@RequestMapping("/weChat")
public class WeChatController {

    private final static String TOKEN = "abcd1234";
    private final static String ENCODING = "UTF-8";
    private final static String ACCESS_TOKEN = "access_token";
    private final static String REFRESH_TOKEN = "refresh_token";
    private final static String OPEN_ID = "openid";
    private final static String NICK_NAME = "nickname";
    private final static String SEX = "sex";
    private final static String HEAD_IMG_URL = "headimgurl";
    /**
     * 二维码保存路径
     */
    private final static String IMG_PATH = "D:\\Users\\code.png";

//    private final WeChatUserService weChatUserService;

    //    public WeChatController(WeChatUserService weChatUserService) {
//        this.weChatUserService = weChatUserService;
//    }
    @Resource
    private WeChatUtil weChatUtil;

    @Resource
    private TWxUserService wxUserService;

    @Resource(name = "commonAsyncExecutor")
    private ThreadPoolTaskExecutor threadPoolTaskExecutor;

    @Autowired
    private CouponReceiveService couponReceiveService;

    @Resource
    private WxTokenService wxTokenService;

    /**
     * 微信Token验证
     *
     * @param signature 微信加密签名
     * @param timestamp 时间戳
     * @param nonce     随机数
     * @param echostr   随机字符串
     * @param response  HTTP响应对象
     * @throws Exception 如果处理过程中出现错误
     */
    @GetMapping("/verifyToken")
    public void verifyToken(@RequestParam(value = "signature") String signature,
                            @RequestParam(value = "timestamp") String timestamp,
                            @RequestParam(value = "nonce") String nonce,
                            @RequestParam(value = "echostr") String echostr, HttpServletResponse response) throws Exception {

        log.info("微信Token验证 入参: signature:{},timestamp:{},nonce:{},echostr:{},", signature, timestamp, nonce, echostr);
        // 参数排序
        String[] params = new String[]{timestamp, nonce, TOKEN};
        Arrays.sort(params);

        // 校验成功则响应 echostr，失败则不响应
        if (verification(params, signature) && echostr != null) {
            response.setCharacterEncoding(ENCODING);
            response.getWriter().write(echostr);
            response.getWriter().flush();
            response.getWriter().close();
        }
    }

    @PostMapping("/verifyToken")
    @Log(title = "公众号pverifyToken", businessType = BusinessType.OTHER)
    public void pverifyToken(@RequestParam(value = "signature") String signature,
                             @RequestParam(value = "timestamp") String timestamp,
                             @RequestParam(value = "nonce") String nonce) {

        log.info("11111111111111111111111111111111111111111111");
        // 参数排序
        String[] params = new String[]{timestamp, nonce, TOKEN};
        Arrays.sort(params);

//        {
//            "signature": "63c8f3e1fb23d0d18f6d7ddf8019dc8a8737a04b",
//                "openid": "oPYgb6qoOUXYIQLbB7f6zOF40kwk",
//                "nonce": "1376426761",
//                "timestamp": "1717569593"
//        }

    }

    /**
     * 获取微信code
     *
     * @param state 状态参数
     */
    @ApiOperation("获取微信code")
    @Log(title = "获取微信code", businessType = BusinessType.OTHER)
    @GetMapping("/getCode")
    public String weiXinLogin(String state) {
//        QrConfig config = new QrConfig(300, 300);
        // 设置边距，即二维码和背景之间的边距
//        config.setMargin(1);
        // 生成二维码到文件，也可以到流
        String code = weChatUtil.getCode(state);
        log.info("code:{}", code);
//        QrCodeUtil.generate(code, config,
//                FileUtil.file(IMG_PATH));
        return code;
    }

    /**
     * 获取token和userInfo
     *
     * @param code 微信授权码
     * @return 访问令牌
     */
    @GetMapping("/getAccessToken")
    public R getAccessToken(@RequestParam String code) {
        // 发送get请求获取 AccessToken
        Map<?, ?> result = weChatUtil.getAccessToken(code);
        String accessToken = result.get(ACCESS_TOKEN).toString();
        String refreshToken = result.get(REFRESH_TOKEN).toString();
        String openid = result.get(OPEN_ID).toString();
//      WeChatUser user = weChatUserService.getOne(new LambdaUpdateWrapper<WeChatUser>().eq(WeChatUser::getOpenid, openid));

        // 如果用户历史上已经完成授权
//        if (user != null) {
//            log.info("该用户已授权");
//            return "<h1>你已经授权过啦~</h1>";
//        }

        // 如果用户是第一次进行微信公众号授权
        // 进行这一步时用户应点击了同意授权按钮
        String userInfoJsom = weChatUtil.getUserInfo(accessToken, openid);
        // 解析JSON数据
        JSONObject jsonObject = new JSONObject(userInfoJsom);
        // 将用户信息保存到数据库中
        LambdaQueryWrapper<TWxUser> objectLambdaQueryWrapper = new LambdaQueryWrapper<>();
        objectLambdaQueryWrapper.eq(TWxUser::getcOpenid, openid);
        TWxUser user = wxUserService.getOne(objectLambdaQueryWrapper);
        if (user == null) {
            user = new TWxUser();
            user.setcOpenid(openid);
            user.setcNickName(jsonObject.get("nickname").toString());
            user.setcIcon(jsonObject.get("headimgurl").toString());
//            user.setcPhone(phoneNumber);
            wxUserService.save(user);
            //异步 添加新人优惠卷
            TWxUser finalUser = user;
            threadPoolTaskExecutor.submit(() -> couponReceiveService.submit(new CouponReceive().setOpenid(finalUser.getcOpenid()).setCouponId("1")));
            user.setId(user.getId());
        }

        WxLoginUser wxUser = new WxLoginUser();
        BeanUtils.copyProperties(user, wxUser);
        // 生成并返回令牌
        String token = wxTokenService.createToken(wxUser);
        if (token == null || token.isEmpty()) {
            return R.fail("生成令牌失败");
        }
        wxUser.setToken(token);
        // 返回用户信息
        // 记录登录信息
        AsyncManager.me().execute(AsyncFactory.recordLogininfor(wxUser.getCOpenid(), Constants.LOGIN_SUCCESS, MessageUtils.message("user.login.success")));
        return R.ok(wxUser);
    }

    /**
     * 刷新token，微信提供的token是有限时间的，但是对于财务报销系统仅需授权一次的情况下一般不需要进行更新
     *
     * @return accessToken
     */
    @GetMapping("/refreshToken")
    public R<String> refreshToken() {
        // TODO 这里需要绑定系统真实的用户id
//        Long userId = StpUtil.getLoginIdAsLong();
        //Long userId = 1L;

//        WeChatUser weChatUser = weChatUserService.getOne(new LambdaUpdateWrapper<WeChatUser>().eq(WeChatUser::getUserId, userId));
//        if (weChatUser == null){
//            return Result.error("error");
//        }

        // 发送get请求获取 RefreshToken
        Map<?, ?> result = weChatUtil.refreshToken("");//weChatUser.getRefreshToken()
        String accessToken = result.get(ACCESS_TOKEN).toString();
        String refreshToken = result.get(REFRESH_TOKEN).toString();

        // 更新用户信息
//        WeChatUser weChatUserUpdate = new WeChatUser();
//        weChatUserUpdate.setId(weChatUser.getId());
//        weChatUserUpdate.setAccessToken(accessToken);
//        weChatUserUpdate.setRefreshToken(refreshToken);
//        weChatUserUpdate.setUpdateDate(LocalDateTime.now());

        // 存储数据库
//        weChatUserService.updateById(weChatUserUpdate);

        return R.ok(accessToken);
    }
}
